"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectReferencedMembers = exports.collectUnreferencedMembersErrors = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
function collectUnreferencedMembersErrors(referencedMembers, unreferencedMembersErrors, file, sourceFile, classDeclaration) {
    for (const member of classDeclaration.members) {
        if (!referencedMembers.has(member) && member.name && typescript_1.default.isIdentifier(member.name)) {
            const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, member.name.getStart(sourceFile));
            unreferencedMembersErrors.push({ file, name: member.name.text, line, character, type: `class ${classDeclaration.name.text} member` });
        }
    }
}
exports.collectUnreferencedMembersErrors = collectUnreferencedMembersErrors;
function collectReferencedMembers(classDeclaration, languageService, file, node) {
    const referencedMembers = new Set();
    const decoratedMembers = new Set();
    for (const member of classDeclaration.members) {
        if (typescript_1.default.isConstructorDeclaration(member)) {
            referencedMembers.add(member);
        }
        else if (member.modifiers
            && member.modifiers.some(m => m.kind === typescript_1.default.SyntaxKind.PublicKeyword
                || m.kind === typescript_1.default.SyntaxKind.PrivateKeyword
                || m.kind === typescript_1.default.SyntaxKind.ProtectedKeyword)) {
            referencedMembers.add(member);
        }
        else if (member.name && typescript_1.default.isIdentifier(member.name)) {
            const identifier = member.name;
            if (hookNames.includes(identifier.text)) {
                referencedMembers.add(member);
            }
            else if (member.decorators) {
                for (const decorator of member.decorators) {
                    if (typescript_1.default.isCallExpression(decorator.expression)) {
                        const { expression } = decorator.expression;
                        if (typescript_1.default.isIdentifier(expression)) {
                            const text = expression.text;
                            if (text === 'Input' || text === 'Output') {
                                referencedMembers.add(member);
                                decoratedMembers.add(identifier.text);
                                break;
                            }
                        }
                    }
                }
            }
            else if (decoratedMembers.has(identifier.text)) {
                referencedMembers.add(member);
            }
            if (!referencedMembers.has(member)) {
                const references = languageService.getReferencesAtPosition(file, identifier.end);
                if (references
                    && references.some(r => r.fileName !== file
                        || r.textSpan.start < node.pos
                        || r.textSpan.start > node.end)) {
                    referencedMembers.add(member);
                }
            }
        }
    }
    return referencedMembers;
}
exports.collectReferencedMembers = collectReferencedMembers;
const hookNames = [
    'beforeCreate', 'created',
    'beforeMount', 'mounted',
    'beforeUpdate', 'updated',
    'activated', 'deactivated',
    'beforeDestroy', 'destroyed',
    'render',
    'componentWillMount', 'componentDidMount', 'UNSAFE_componentWillMount',
    'componentWillReceiveProps', 'UNSAFE_componentWillReceiveProps',
    'shouldComponentUpdate',
    'componentWillUpdate', 'componentDidUpdate', 'UNSAFE_componentWillUpdate',
    'componentWillUnmount',
    'ngOnInit',
    'ngOnChanges',
    'ngDoCheck',
    'ngOnDestroy',
    'ngAfterContentInit',
    'ngAfterContentChecked',
    'ngAfterViewInit',
    'ngAfterViewChecked'
];
