"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectUnusedExportsErrors = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
function collectUnusedExportsErrors(node, file, sourceFile, languageService, unusedExportsErrors) {
    if (node.modifiers && node.modifiers.some(m => m.kind === typescript_1.default.SyntaxKind.ExportKeyword)) {
        const jsDocs = getJsDocs(node);
        const isPublic = jsDocs.find(jsDoc => ['public', 'internal', 'alpha', 'beta'].includes(jsDoc.name));
        if (!isPublic) {
            if (typescript_1.default.isTypeAliasDeclaration(node)) {
                collectErrors(file, node.name, sourceFile, 'type', languageService, unusedExportsErrors);
            }
            else if (typescript_1.default.isFunctionDeclaration(node)) {
                collectErrors(file, node.name, sourceFile, 'function', languageService, unusedExportsErrors);
            }
            else if (typescript_1.default.isClassDeclaration(node)) {
                collectErrors(file, node.name, sourceFile, 'class', languageService, unusedExportsErrors);
            }
            else if (typescript_1.default.isInterfaceDeclaration(node)) {
                collectErrors(file, node.name, sourceFile, 'interface', languageService, unusedExportsErrors);
            }
            else if (typescript_1.default.isVariableStatement(node)) {
                for (const declaration of node.declarationList.declarations) {
                    if (typescript_1.default.isIdentifier(declaration.name)) {
                        collectErrors(file, declaration.name, sourceFile, 'variable', languageService, unusedExportsErrors);
                    }
                }
            }
        }
    }
}
exports.collectUnusedExportsErrors = collectUnusedExportsErrors;
function collectErrors(file, identifier, sourceFile, type, languageService, unusedExportsErrors) {
    if (identifier) {
        const references = languageService.getReferencesAtPosition(file, identifier.end);
        if (references && references.every(r => r.fileName === file)) {
            const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, identifier.getStart(sourceFile));
            unusedExportsErrors.push({ file, name: identifier.text, line, character, type });
        }
    }
}
function getJsDocs(node) {
    const jsDocs = node.jsDoc;
    const result = [];
    if (jsDocs && jsDocs.length > 0) {
        for (const jsDoc of jsDocs) {
            if (jsDoc.tags) {
                for (const tag of jsDoc.tags) {
                    result.push({
                        name: tag.tagName.text,
                        comment: typeof tag.comment === 'string' ? tag.comment : undefined
                    });
                }
            }
        }
    }
    return result;
}
