"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectMissingKeyErrors = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
const parse5 = (0, tslib_1.__importStar)(require("parse5"));
function collectMissingKeyErrors(propertyName, propertyInitialize, templateText, missingKeyErrors, file, sourceFile) {
    if (templateText) {
        const fragment = parse5.parseFragment(templateText);
        const errorCount = keyExistsInNode(0, fragment);
        if (errorCount > 0) {
            const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, propertyInitialize.getStart(sourceFile));
            missingKeyErrors.push({ file, name: propertyName, line, character, type: `'v-for' or '*ngFor'(error count: ${errorCount})` });
        }
    }
}
exports.collectMissingKeyErrors = collectMissingKeyErrors;
function keyExistsInNode(errorCount, node) {
    if (node.nodeName.startsWith('#')) {
        if (node.nodeName === '#document-fragment') {
            for (const childNode of node.childNodes) {
                errorCount = keyExistsInNode(errorCount, childNode);
            }
        }
    }
    else {
        const elementNode = node;
        if (elementNode.tagName !== 'template' && elementNode.attrs) {
            const angularAttr = elementNode.attrs.find(attr => attr.name === '*ngfor');
            if (angularAttr && (!angularAttr.value || !angularAttr.value.includes('trackBy'))) {
                errorCount++;
            }
            const vueAttr = elementNode.attrs.find(attr => attr.name === 'v-for');
            if (vueAttr && elementNode.attrs.every(attr => attr.name !== 'key' && attr.name !== ':key')) {
                errorCount++;
            }
        }
        if (elementNode.childNodes) {
            for (const childNode of elementNode.childNodes) {
                errorCount = keyExistsInNode(errorCount, childNode);
            }
        }
        const content = elementNode.content;
        if (content) {
            errorCount = keyExistsInNode(errorCount, content);
        }
    }
    return errorCount;
}
