"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectPromiseNotAwaitErrors = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
function collectPromiseNotAwaitErrors(node, checker, promiseNotAwaitErrors, file, sourceFile) {
    if (typescript_1.default.isCallExpression(node)) {
        if (typescript_1.default.isReturnStatement(node.parent) || typescript_1.default.isAwaitExpression(node.parent)) {
            return;
        }
        const functionNode = findFunction(node);
        if (!functionNode || !functionNode.modifiers || functionNode.modifiers.every((m) => m.kind !== typescript_1.default.SyntaxKind.AsyncKeyword)) {
            return;
        }
        checkCallExpressionReturnPromise(node, checker, promiseNotAwaitErrors, file, sourceFile);
    }
    else if (typescript_1.default.isIfStatement(node) || typescript_1.default.isWhileStatement(node)) {
        if (typescript_1.default.isCallExpression(node.expression)) {
            checkCallExpressionReturnPromise(node.expression, checker, promiseNotAwaitErrors, file, sourceFile);
        }
        else if (typescript_1.default.isIdentifier(node.expression)) {
            const type = checker.getTypeAtLocation(node.expression);
            checkTypeIsPromise(node, type, promiseNotAwaitErrors, file, sourceFile);
        }
    }
}
exports.collectPromiseNotAwaitErrors = collectPromiseNotAwaitErrors;
function checkCallExpressionReturnPromise(node, checker, promiseNotAwaitErrors, file, sourceFile) {
    const signature = checker.getResolvedSignature(node);
    if (signature) {
        const returnType = checker.getReturnTypeOfSignature(signature);
        checkTypeIsPromise(node, returnType, promiseNotAwaitErrors, file, sourceFile);
    }
}
function checkTypeIsPromise(node, type, promiseNotAwaitErrors, file, sourceFile) {
    if (type.symbol && type.symbol.escapedName === 'Promise') {
        const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, node.getStart(sourceFile));
        const text = typescript_1.default.isIdentifier(node.expression) ? node.expression.text : '';
        promiseNotAwaitErrors.push({ file, name: text, line, character, type: '' });
    }
}
function findFunction(node) {
    return findParentFunction(node);
}
function findParentFunction(node) {
    const parent = node.parent;
    if (typescript_1.default.isFunctionDeclaration(parent)
        || typescript_1.default.isFunctionExpression(parent)
        || typescript_1.default.isArrowFunction(parent)
        || typescript_1.default.isMethodDeclaration(parent)) {
        return parent;
    }
    if (typescript_1.default.isSourceFile(parent)) {
        return undefined;
    }
    return findParentFunction(parent);
}
