"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAngularAttrName = exports.collectCanOnlyBePublicErrors = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
const parse5 = (0, tslib_1.__importStar)(require("parse5"));
function collectCanOnlyBePublicErrors(members, referencedMembers, templateText, canOnlyBePublicErrors, file, sourceFile, classDeclaration) {
    if (templateText && members.length > 0) {
        const fragment = parse5.parseFragment(templateText);
        for (const member of members) {
            const identifier = member.name;
            if (identifier) {
                const templateType = memberIsUsedInNode(identifier.text, fragment);
                if (templateType) {
                    if (!referencedMembers.has(member)) {
                        referencedMembers.add(member);
                    }
                    if (templateType !== "vue" /* vue */
                        && member.modifiers
                        && member.modifiers.some(m => m.kind === typescript_1.default.SyntaxKind.PrivateKeyword
                            || m.kind === typescript_1.default.SyntaxKind.ProtectedKeyword)) {
                        const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, identifier.getStart(sourceFile));
                        canOnlyBePublicErrors.push({ file, name: identifier.text, line, character, type: `class ${classDeclaration.name.text} member` });
                    }
                }
            }
        }
    }
}
exports.collectCanOnlyBePublicErrors = collectCanOnlyBePublicErrors;
function memberIsUsedInNode(memberName, node) {
    if (node.nodeName.startsWith('#')) {
        if (node.nodeName === '#text') {
            const textNode = node;
            return !!textNode.value
                && textNode.value.includes(memberName)
                && !new RegExp(`{{.*'.*${memberName}.*'.*}}`).test(textNode.value)
                && !new RegExp(`{{.*".*${memberName}.*".*}}`).test(textNode.value);
        }
        else if (node.nodeName === '#document-fragment') {
            for (const childNode of node.childNodes) {
                const isUsed = memberIsUsedInNode(memberName, childNode);
                if (isUsed) {
                    return isUsed;
                }
            }
            return false;
        }
    }
    else {
        const elementNode = node;
        if (elementNode.attrs) {
            for (const attr of elementNode.attrs) {
                let templateType;
                if (isVuejsAttrName(attr.name)) {
                    templateType = "vue" /* vue */;
                }
                else if (isAngularAttrName(attr.name)) {
                    templateType = "angular" /* angular */;
                }
                if (templateType) {
                    const isUsed = attr.value
                        && attr.value.includes(memberName)
                        && !new RegExp(`'.*${memberName}.*'`).test(attr.value)
                        && !new RegExp(`".*${memberName}.*"`).test(attr.value);
                    if (isUsed) {
                        return templateType;
                    }
                }
            }
        }
        if (elementNode.childNodes) {
            for (const childNode of elementNode.childNodes) {
                const isUsed = memberIsUsedInNode(memberName, childNode);
                if (isUsed) {
                    return isUsed;
                }
            }
        }
        const content = elementNode.content;
        if (content) {
            return memberIsUsedInNode(memberName, content);
        }
    }
    return false;
}
function isVuejsAttrName(attrName) {
    return attrName.startsWith('v-')
        || attrName.startsWith(':')
        || attrName.startsWith('@');
}
function isAngularAttrName(attrName) {
    return attrName.startsWith('*ng')
        || (attrName.startsWith('[') && attrName.endsWith(']'))
        || (attrName.startsWith('(') && attrName.endsWith(')'));
}
exports.isAngularAttrName = isAngularAttrName;
